/*
 * A231216 Demo: LCD contrast with PWM
 * 
 *
 *
 */

// HW definitions (ref. HW: A231216))
#define _XTAL_FREQ 4000000
#define RS RB5
#define EN RB4
#define D4 RA4
#define D5 RA6
#define D6 RA7
#define D7 RB0

// Fuses Config
#pragma config FOSC = INTOSCIO  // Oscillator Selection bits (INTOSC oscillator: I/O function on RA6/OSC2/CLKOUT pin, I/O function on RA7/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable bit (PWRT enabled)
#pragma config MCLRE = ON       // RA5/MCLR/VPP Pin Function Select bit (RA5/MCLR/VPP pin function is MCLR)
#pragma config BOREN = ON       // Brown-out Detect Enable bit (BOD enabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable bit (RB4/PGM pin has digital I/O function, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EE Memory Code Protection bit (Data memory code protection off)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)

#include <xc.h>
#include <stdio.h>
#include "lcd.h"

void PWM_Duty(unsigned int duty);
char KEY1();
char KEY2();

void main()
{
    unsigned int Duty = (unsigned int)((1.1/5)* 1023); // Set PWM to produce  1.1V on V0
    char string[16];
    
    CMCON = 0b00000111;         // Disable comparators to enable RA0-RA3 input
    TRISA = 0b00101110;         // Set RA1-RA3 and RA5 as input, others as output
    TRISB = 0b11000110;         // Set RB1-RB2 and RB6-RB7 as input, others as output
    RA0 = 1;                    // LED1 OFF
    
    // Set PWM for LCD contrast control voltage (V0)
    /*
    * Fosc = 4000000 Hz
    * Fpwm = 1000.00 Hz
    * Resolution is 10 bits
    * Prescaler is 4
    */
    PR2 = 0b11111001 ;
    T2CON = 0b00000101 ;
    CCP1CON = 0b00001100 ;
    PWM_Duty(Duty);
        
    Lcd_Init();
    Lcd_Clear();
    Lcd_Set_Cursor(1,1);
    Lcd_Write_String((char *)" A231216 PIC16F");
    Lcd_Set_Cursor(2,1);
    Lcd_Write_String((char *)"   Demo Board");
    
    while(1)
    {
        if (KEY1() == 1) 
        {
            if (Duty < 1023) Duty++;
            PWM_Duty(Duty);
            sprintf(string, "-> CNTR- = %4u", Duty);
            Lcd_Set_Cursor(2,1);
            Lcd_Write_String(string);
            __delay_ms(10);
        }
        else
        {  
            if (KEY2() == 1)
            {
                if (Duty > 0) Duty--;
                PWM_Duty(Duty);
                sprintf(string, "-> CNTR+ = %4u", Duty);
                Lcd_Set_Cursor(2,1);
                Lcd_Write_String(string);
                __delay_ms(10);
            }
            else
            {
                Lcd_Set_Cursor(2,1);
                Lcd_Write_String((char *)"   Demo Board   ");
            }
        }
    }
}

void PWM_Duty(unsigned int duty)
{
  if (duty < 1024)
  {
    CCP1Y = duty & 1;
    CCP1X = duty & 2;
    CCPR1L = duty >> 2;
  }
}

char KEY1()
{
    char Key1Pressed = 0;
    TRISA0 = 1;                         // RA0 as input
    if (RA0 == 0) Key1Pressed = 1;      // Key1 is pressed (RA0 = LOW)?
    TRISA0 = 0;                         // RA0 as output
    return Key1Pressed;
}

char KEY2()
{
    char Key2Pressed = 0;
    if (RA1 == 0) Key2Pressed = 1;      // Key1 is pressed (RA0 = LOW)?
    return Key2Pressed;
}

