/*
 * A231216 Demo: PWM LED (pins 2-3 of connector J4 "GPIO1/LCD")
 * 
 *
 *
 */

#define _XTAL_FREQ 4000000

// Fuses Config
#pragma config FOSC = INTOSCIO  // Oscillator Selection bits (INTOSC oscillator: I/O function on RA6/OSC2/CLKOUT pin, I/O function on RA7/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable bit (PWRT enabled)
#pragma config MCLRE = ON       // RA5/MCLR/VPP Pin Function Select bit (RA5/MCLR/VPP pin function is MCLR)
#pragma config BOREN = ON       // Brown-out Detect Enable bit (BOD enabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable bit (RB4/PGM pin has digital I/O function, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EE Memory Code Protection bit (Data memory code protection off)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)

#include <xc.h>

void PWM_Duty(unsigned int duty);

void main()
{
    unsigned int Duty = (unsigned int)((1.1/5)* 1023); // Set PWM to produce  1.1V on V0
    char string[16];
    
    CMCON = 0b00000111;         // Disable comparators to enable RA0-RA3 input
    TRISA = 0b00101110;         // Set RA1-RA3 and RA5 as input, others as output
    TRISB = 0b11000110;         // Set RB1-RB2 and RB6-RB7 as input, others as output
    RA0 = 1;                    // LED1 OFF
    
    // Set PWM for LCD contrast control voltage (V0)
    /*
    * Fosc = 4000000 Hz
    * Fpwm = 1000.00 Hz
    * Resolution is 10 bits
    * Prescaler is 4
    */
    PR2 = 0b11111001 ;
    T2CON = 0b00000101 ;
    CCP1CON = 0b00001100 ;
    PWM_Duty(Duty);
        
    while(1)
    {
        RA0 = 0;                // LED1 ON  
        for (int i = 1024; i >= 0; i--)
        {
            PWM_Duty(i);
            __delay_ms(1);
        }
        RA0 = 1;                // LED1 OFF
        for (int i = 0; i < 1024; i++)
        {
            PWM_Duty(i);
            __delay_ms(1);
        }
    }
}

void PWM_Duty(unsigned int duty)
{
  if (duty < 1024)
  {
    CCP1Y = duty & 1;
    CCP1X = duty & 2;
    CCPR1L = duty >> 2;
  }
}