/*
 * A231216 Demo: Serial to LCD
 * 
 *
 *
 */

#define _XTAL_FREQ 4000000
#define RS RB5
#define EN RB4
#define D4 RA4
#define D5 RA6
#define D6 RA7
#define D7 RB0

// CONFIG
#pragma config FOSC = INTOSCIO  // Oscillator Selection bits (INTOSC oscillator: I/O function on RA6/OSC2/CLKOUT pin, I/O function on RA7/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable bit (PWRT enabled)
#pragma config MCLRE = ON       // RA5/MCLR/VPP Pin Function Select bit (RA5/MCLR/VPP pin function is MCLR)
#pragma config BOREN = ON       // Brown-out Detect Enable bit (BOD enabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable bit (RB4/PGM pin has digital I/O function, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EE Memory Code Protection bit (Data memory code protection off)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)

#include <xc.h>
#include "lcd.h"

long freq;

void UART_Write(char data);
char UART_Read();
void PWM_Duty(unsigned int duty);

void main()
{
    char rx_char;
    int line;
    
    CMCON = 0b00000111;             // Disable comparators to enable RA0-RA3 input
    TRISA = 0b00101110;             // Set RA1-RA3 and RA5 as input, others as output
    TRISB = 0b11000110;             // Set RB1-RB2 and RB6-RB7 as input, others as output
    nRBPU = 0;                      // Enable PORTB Internal Pull Up Resistors
    RA0 = 1;                        // LED1 OFF
    
    // Configure UART
    SPBRG = 25;                     // 9600 BAUD
    BRGH = 1;                       // High speed baud rate
    TX9 = 0;                        // Tx 8 bit transmission
    TXEN = 1;                       // Tx Transmit enable
    SYNC = 0;                       // Async mode
    SPEN = 1;                       // Serial port enabled
    RX9 = 0;                        // Rx 8 bit mode
    CREN = 1;                       // Rx Enable receive

    // Configure and start PWM for VO generation (LCD contrast control voltage)   
    PR2 = 0b11111001 ;
    T2CON = 0b00000101 ;
    CCP1CON = 0b00001100 ;
    PWM_Duty((unsigned int)((1.1/5)* 1023)); // Set PWM to produce  1.1V on V0);
    
    Lcd_Init();
    Lcd_Clear();
    Lcd_Set_Cursor(1,1);
    Lcd_Write_String((char *)" A231216 PIC16F");
    Lcd_Set_Cursor(2,1);
    Lcd_Write_String((char *)"   Demo Board");
    __delay_ms(2000);
            
    Lcd_Clear();
    Lcd_Set_Cursor(1,1);
    Lcd_Write_String((char *)"  Waiting for");
    Lcd_Set_Cursor(2,1);
    Lcd_Write_String((char *)"  serial I/O...");
    
    while(!RCIF);                   // Wait for a char from serial
    Lcd_Clear();
    Lcd_Set_Cursor(1,1);
    line = 1;
    
    while(1)
    {
        rx_char = UART_Read();
        UART_Write(rx_char);
        if (rx_char == 0x0D)        // Received a CR?
        {
            UART_Write(0x0A);       // Tx a LF
            line++;
            if (line > 2) line = 1;
            Lcd_Set_Cursor(line,1);
        }
        else
            Lcd_Write_Char(rx_char);
    }
}

void UART_Write(char data)
{
  while(!TRMT);
  TXREG = data;
}

char UART_Read()
{
  while(!RCIF);                     // Wait for a received char
  
  // Blink LED1 every received char
  RA0 = 0;
  __delay_ms(4);
  RA0 = 1;
  
  return RCREG;
}

void PWM_Duty(unsigned int duty)
{
  if (duty < 1024)
  {
    CCP1Y = duty & 1;
    CCP1X = duty & 2;
    CCPR1L = duty >> 2;
  }
}

